/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.search.test.util;

import com.liferay.portal.kernel.search.SearchContext;
import com.liferay.portal.kernel.search.facet.Facet;
import com.liferay.portal.kernel.search.facet.collector.FacetCollector;
import com.liferay.portal.kernel.search.facet.collector.TermCollector;

import java.util.List;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import org.junit.Assert;

/**
 * @author André de Oliveira
 */
public class FacetsAssert {

	public static void assertFrequencies(
		String facetName, SearchContext searchContext, List<String> expected) {

		Facet facet = searchContext.getFacet(facetName);

		FacetCollector facetCollector = facet.getFacetCollector();

		List<TermCollector> termCollectors = facetCollector.getTermCollectors();

		Assert.assertNotNull(termCollectors);

		Assert.assertEquals(
			(String)searchContext.getAttribute("queryString"),
			expected.toString(),
			_toString(
				termCollectors,
				termCollector ->
					termCollector.getTerm() + "=" +
						termCollector.getFrequency()));
	}

	private static <T> String _toString(
		List<? extends T> list, Function<? super T, String> function) {

		Stream<? extends T> stream = list.stream();

		return stream.map(
			function
		).collect(
			Collectors.toList()
		).toString();
	}

}