/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.search.test.util;

import java.util.concurrent.Callable;
import java.util.concurrent.TimeUnit;

/**
 * @author André de Oliveira
 */
public class IdempotentRetryAssert {

	public static <T> T retryAssert(
			long timeout, TimeUnit timeoutTimeUnit, Callable<T> callable)
		throws Exception {

		return retryAssert(
			timeout, timeoutTimeUnit, 0, TimeUnit.SECONDS, callable);
	}

	public static <T> T retryAssert(
			long timeout, TimeUnit timeoutTimeUnit, long pause,
			TimeUnit pauseTimeUnit, Callable<T> callable)
		throws Exception {

		long deadline =
			System.currentTimeMillis() + timeoutTimeUnit.toMillis(timeout);

		while (true) {
			try {
				return callable.call();
			}
			catch (AssertionError ae) {
				if (System.currentTimeMillis() > deadline) {
					throw ae;
				}
			}

			Thread.sleep(pauseTimeUnit.toMillis(pause));
		}
	}

}