/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.search.test.util.facet;

import com.liferay.portal.kernel.json.JSONArray;
import com.liferay.portal.kernel.search.Field;
import com.liferay.portal.kernel.search.SearchContext;
import com.liferay.portal.kernel.search.facet.MultiValueFacet;
import com.liferay.portal.kernel.util.StringUtil;

import java.util.Arrays;

import org.junit.Test;

/**
 * @author André de Oliveira
 */
public abstract class BaseMultiValueFacetTestCase extends BaseFacetTestCase {

	@Test
	public void testSelection() throws Exception {
		addDocuments(6, "one");
		addDocuments(5, "two");
		addDocuments(4, "three");
		addDocuments(3, "four");
		addDocuments(2, "five");

		assertSearchFacet(
			helper -> {
				MultiValueFacet multiValueFacet = helper.addFacet(
					this::createFacet);

				select(multiValueFacet, helper, "three", "one");

				helper.search();

				helper.assertResultCount(10);

				helper.assertFrequencies(
					multiValueFacet,
					Arrays.asList(
						"one=6", "two=5", "three=4", "four=3", "five=2"));
			});
	}

	protected MultiValueFacet createFacet(SearchContext searchContext) {
		MultiValueFacet multiValueFacet = new MultiValueFacet(searchContext);

		initFacet(multiValueFacet);

		multiValueFacet.setFieldName(Field.STATUS);

		return multiValueFacet;
	}

	@Override
	protected String getField() {
		return Field.STATUS;
	}

	protected void select(
		MultiValueFacet multiValueFacet, FacetTestHelper facetTestHelper,
		String... values) {

		JSONArray jsonArray = jsonFactory.createJSONArray();

		for (String value : values) {
			jsonArray.put(value);
		}

		multiValueFacet.setValues(jsonArray);

		facetTestHelper.setSearchContextAttribute(
			multiValueFacet.getFieldId(), StringUtil.merge(values));
	}

}