/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

package com.liferay.portal.search.test.util.indexing;

import com.liferay.portal.kernel.search.BooleanClauseOccur;
import com.liferay.portal.kernel.search.Query;
import com.liferay.portal.kernel.search.generic.MatchQuery;
import com.liferay.portal.kernel.search.generic.TermQueryImpl;

/**
 * @author André de Oliveira
 */
public class QueryContributors {

	public static QueryContributor dummy() {
		return booleanQuery -> {
		};
	}

	public static QueryContributor must(Query query) {
		return booleanQuery -> QueryContributor.add(
			booleanQuery, query, BooleanClauseOccur.MUST);
	}

	public static QueryContributor mustMatch(String field, String value) {
		return must(new MatchQuery(field, value));
	}

	public static QueryContributor mustNot(Query query) {
		return booleanQuery -> QueryContributor.add(
			booleanQuery, query, BooleanClauseOccur.MUST_NOT);
	}

	public static QueryContributor mustNotMatch(String field, String value) {
		return mustNot(new MatchQuery(field, value));
	}

	public static QueryContributor mustNotTerm(String field, String value) {
		return mustNot(new TermQueryImpl(field, value));
	}

	public static QueryContributor mustTerm(String field, String value) {
		return must(new TermQueryImpl(field, value));
	}

}